<?php
/***************************************************************************
 *   @brief WP-Sentinel - Wordpress Security System .                      *
 *   @author Simone Margaritelli (aka evilsocket) <evilsocket@gmail.com>   *
 *                       		                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

class WPS_Report {
	private $email_row_layout;
	private $email_layout;
	private $alarm_row_layout;
	private $alarm_layout;
	private $alarms;
	private $wp_user;
	private $user;
	
	public function __construct( $wpuser ){
		$this->email_row_layout = @file_get_contents(WPS_PATH."layouts/email.row.layout");
		$this->email_layout     = @file_get_contents(WPS_PATH."layouts/email.layout");
		$this->alarm_row_layout = @file_get_contents(WPS_PATH."layouts/alarm.row.layout");
		$this->alarm_layout     = @file_get_contents(WPS_PATH."layouts/alarm.layout");

		$this->alarms  = array();
		$this->wp_user = $wpuser;
		$this->user    = array();
		
		$this->user['address'] = wp_sentinel_get_address();
		$this->user['referer'] = htmlentities($_SERVER['HTTP_REFERER']);
		$this->user['agent']   = htmlentities($_SERVER['HTTP_USER_AGENT']);
	}

	public function addAlarm( $scope, $variable, $content, $alarm, $rule ){
		$this->alarms[] = array( "scope"    => $scope,
		                         "variable" => htmlentities($variable),
		 						 "content"  => htmlentities($content),
								 "alarm"    => $alarm,
								 "rule"     => htmlentities($rule) );
	}
	
	public function hasAlarms(){
		return (count($this->alarms) > 0);
	}
	
	public function alarms(){
		return $this->alarms;
	}
	
	public function logReport(){
		$report = "";
		foreach( $this->alarms as $alarm ){
			$report .= time()." : {$this->user[address]} ".
					   (isset($this->wp_user->user_level) ? "({$this->wp_user->user_login}) " : "").
					   ": {$this->user[referer]} : {$this->user[agent]} : ".
			           "{$alarm[scope]} : {$alarm[variable]} : ".@base64_encode($alarm['content'])." : {$alarm[alarm]}\r\n";
		}
		return $report;
	}
	
	public function emailReport(){
		$rows = '';
		foreach( $this->alarms as $alarm ){
			$entry = $this->email_row_layout;
			foreach( array_keys($alarm) as $key ){
				$token = "{".strtoupper($key)."}";
				$entry = str_replace( $token, $alarm[$key], $entry );
			}
			$rows .= $entry;
		}

		$report = str_replace( "{ALARM_ROWS}", $rows, $this->email_layout );
		$report = str_replace( "{DATE}", 	   strftime("%c"), $report );
		$report = str_replace( "{ADDRESS}",    $this->user[address], $report );
		$report = str_replace( "{USERNAME}",   (isset($this->wp_user->user_level) ? $this->wp_user->user_login : ""), $report );
		$report = str_replace( "{REFERER}",    $this->user[referer], $report );
		$report = str_replace( "{USERAGENT}",  $this->user[agent], $report );

		return $report;
	}
	
	private function userReportEntry( $alarm ){
		$entry = $this->alarm_row_layout;
		foreach( array_keys($alarm) as $key ){
			$token = "{".strtoupper($key)."}";
			$entry = str_replace( $token, $alarm[$key], $entry );
		}
		
		return $entry;
	}
	
	public function userReport(){
		$rows = '';
		foreach( $this->alarms as $alarm ){
			$rows .= $this->userReportEntry($alarm);
		}
	
		$report = str_replace( "{ALARM_ROWS}", $rows, $this->alarm_layout );
		
		return $report;
	}
}
	
?>