<?php
/*
Plugin Name: WP-Sentinel
Plugin URI: http://www.evilsocket.net/
Version: 1.0.8.3
Author: Simone Margaritelli aka evilsocket
Description: WordPress security system.
*/
/***************************************************************************
 *   @brief WP-Sentinel - Wordpress Security System .                      *
 *   @author Simone Margaritelli (aka evilsocket) <evilsocket@gmail.com>   *
 *                       		                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

define( "WPS_PATH", ABSPATH."wp-content/plugins/wp-sentinel/" );
define( "WPS_VERS", "1.0.8.3" );

register_activation_hook( WPS_PATH."wp-sentinel.php", "wp_sentinel_activate" );

add_action( 'init', 	           'wp_sentinel_run', 0 );
add_action( 'admin_menu',          'wp_sentinel_menu' );
add_action( 'admin_print_scripts', 'wp_sentinel_enqueue_scripts' );
add_action( 'admin_print_styles',  'wp_sentinel_enqueue_styles' );

include_once( "php/rules.php" );
include_once( "php/config.php" );
include_once( "php/report.php" );
include_once( "php/log.php" );
include_once( "php/settings.php" );

/*
 * Create WP-Sentinel tables.
 */
function wp_sentinel_create_tables( $overwrite = true ){
	global $wpdb;
	
	require_once( ABSPATH.'wp-admin/includes/upgrade.php' );

	$sql = "CREATE TABLE ".($overwrite == true ? '' : 'IF NOT EXISTS')." ".$wpdb->prefix."wps_logs (
	  			id mediumint(9) NOT NULL AUTO_INCREMENT,
	  			timestamp BIGINT(11) DEFAULT '0' NOT NULL,
	  			address VARCHAR(15) NOT NULL,
	  			location VARCHAR(5) NOT NULL,
	  			agent VARCHAR(255) NOT NULL,
	  			referer VARCHAR(255) NOT NULL,
	  			username VARCHAR(25) NOT NULL,
	  			scope VARCHAR(25) NOT NULL,
	  			variable VARCHAR(50) NOT NULL,
				content TEXT NOT NULL,
				message TEXT NOT NULL,
				rule VARCHAR(50) NOT NULL,
	  			UNIQUE KEY id (id)
			);";

    $wpdb->query($sql);
    	
    $sql = "CREATE TABLE ".($overwrite == true ? '' : 'IF NOT EXISTS')." ".$wpdb->prefix."wps_logins (
  				id mediumint(9) NOT NULL AUTO_INCREMENT,
  				timestamp BIGINT(11) DEFAULT '0' NOT NULL,
  				address VARCHAR(15) NOT NULL,
  				attempts INT NOT NULL,
  				UNIQUE KEY id (id)
			);";

    $wpdb->query($sql);
} 

/*
 * Create and preprocess requests environment for further elaborations.
 */
function wp_sentinel_env_preprocessor(){
	$env = array( "GET"    => $_GET,
	              "POST"   => $_POST,
				  "COOKIE" => $_COOKIE,
				  "USER"   => wp_get_current_user() );
				  
	foreach( array( "GET", "POST", "COOKIE" ) as $scope ){
		$data = $env[$scope];
		foreach( $data as $name => $value ){
			/* if magic quotes are on, remove slashes from value */
			if( function_exists('get_magic_quotes_gpc') && get_magic_quotes_gpc() ){
				$value = stripslashes($value);
			}
			/* urldecode the value */
			$value = @urldecode($value);
			/* trim it */
			$value = trim($value);
			/* update the environment */
			$env[$scope][$name] = $value;			
		}
	}
	
	return $env;
}

/*
 * Perform cross site request forgery check upon admin panel.
 */
function wp_sentinel_csrf_check( $env, $report ){
	/* admin panel ? */
	if( preg_match( "/\/wp\-admin\//i", $_SERVER["REQUEST_URI"] ) ){
		/* check if there's a POST request */
		if( count( array_diff( array_values($env['POST']), array('') ) ) ){
			$valid_referer = get_bloginfo('wpurl');
			$rule          = "^".preg_quote( $valid_referer, '/' );
			/* referer not set or not allowed to POST to the admin panel */
			if( isset($_SERVER['HTTP_REFERER']) == false || preg_match( "/$rule/i", $_SERVER['HTTP_REFERER'] ) == false ){
				$report->addAlarm( "HTTP_REFERER", 
				                   "HTTP_REFERER", 
								   $_SERVER['HTTP_REFERER'], 
								   "invalid or null referer for wordpress administration panel request", 
								   $rule );
			}
		}
	}
}

/*
 * Perform anti-flood checking.
 */
function wp_sentinel_is_flooding(){
	global $wps_config, $wpdb;;

	$timestamp = $wpdb->get_var( $wpdb->prepare( "SELECT MAX(timestamp) FROM ".$wpdb->prefix."wps_logs WHERE address = %s", wp_sentinel_get_address() ) );
	if( $timestamp != NULL ){
		$timestamp = (INT)$timestamp;
		$now	   = time();
		$delay	   = $now - $timestamp;
		/* yep, we are getting flooded! */
		if( $delay < $wps_config['flood_time'] ){
			return true;
		}
	}

	return false;	
}
 
/*
 * Upon XSS rule, perform tag checking.
 */
function wp_sentinel_allowed_tags( $value ){
	global $wps_config;
	
	if( preg_match_all( "/<\s*\/?\s*([^ >]+)[^>]*>/m", $value, $tags ) ){
		$tags		 = $tags[1];
		/* any unallowed tags? */
		if( array_intersect( $tags, $wps_config['allowed_html'] ) != $tags ){
			return false;
		}
	}
	return true;
}

function wp_sentinel_allowed_file( $rule, $value ){
	$remotefile = trim($value);
	$hash		= md5($remotefile);
	$cachedfile = WPS_PATH."vectors/$hash.dat";
	$content	= null;
	
	/* file already cached :) */
	if( file_exists($cachedfile) ){
		$content = @file_get_contents($cachedfile);
	}
	/* fetch the file and add it to the cache */
	else{
		$content = @file_get_contents($remotefile);
		@file_put_contents( $cachedfile, $content );
	}
			
	return !preg_match( $rule['classifier'], $content );
}

/*
 * Check if an ip is valid, credits to http://algorytmy.pl/doc/php/function.getenv.php
 */
function wp_sentinel_is_valid_ip( $ip ){
	if (!empty($ip) && ip2long($ip)!=-1) {
    	$reserved_ips = array (
    		array('0.0.0.0','2.255.255.255'),
       		array('10.0.0.0','10.255.255.255'),
       		array('127.0.0.0','127.255.255.255'),
       		array('169.254.0.0','169.254.255.255'),
       		array('172.16.0.0','172.31.255.255'),
       		array('192.0.2.0','192.0.2.255'),
       		array('192.168.0.0','192.168.255.255'),
       		array('255.255.255.0','255.255.255.255')
       	);

       	foreach( $reserved_ips as $r ){
        	$min = ip2long($r[0]);
        	$max = ip2long($r[1]);
        	if ((ip2long($ip) >= $min) && (ip2long($ip) <= $max)) return false;
       	}
       	return true;
	} 
	else { 
		return false;
	}
}

/*
 * Return the real ip address of the remote client.
 */
function wp_sentinel_get_address() {
	if( wp_sentinel_is_valid_ip($_SERVER["HTTP_CLIENT_IP"]) ){
 		return $_SERVER["HTTP_CLIENT_IP"];
	}
	foreach( explode( ",", $_SERVER["HTTP_X_FORWARDED_FOR"] ) as $ip ){
 		if( wp_sentinel_is_valid_ip(trim($ip)) ){
  			return $ip;
 		}
	}
	if( wp_sentinel_is_valid_ip($_SERVER["HTTP_X_FORWARDED"]) ){
 		return $_SERVER["HTTP_X_FORWARDED"];
	} 
	else if( wp_sentinel_is_valid_ip($_SERVER["HTTP_FORWARDED_FOR"]) ){
 		return $_SERVER["HTTP_FORWARDED_FOR"];
	} 
	else if( wp_sentinel_is_valid_ip($_SERVER["HTTP_FORWARDED"]) ){
 		return $_SERVER["HTTP_FORWARDED"];
	} 
	else if( wp_sentinel_is_valid_ip($_SERVER["HTTP_X_FORWARDED"]) ){
 		return $_SERVER["HTTP_X_FORWARDED"];
	} 
	else {
 		return $_SERVER["REMOTE_ADDR"];
	}
}
 
/*
 * Check upon login bruteforcing.
 */
function wp_sentinel_check_bruteforce(){
	global $wps_config, $wpdb;
	
	/* no login attempt, therefore no bruteforce */
	if( !isset($_POST['log']) || !isset($_POST['pwd']) ){
		return false;
	}
	
	$address = wp_sentinel_get_address();
	$attempt = $wpdb->get_row( $wpdb->prepare("SELECT id, timestamp, attempts FROM ".$wpdb->prefix."wps_logins WHERE address = %s", $address ) );
	
	/* first login attempt, create the record and return */
	if( !$attempt ){
		$wpdb->query( $wpdb->prepare("INSERT INTO ".$wpdb->prefix."wps_logins (timestamp,address,attempts) VALUES ( %d, %s, 1 );", time(), $address ) );
		return false;
	}
	/* threshold not exceeded, update the file and return */
	else if( $attempt->attempts < $wps_config['max_logins'] ){
		$wpdb->query( $wpdb->prepare("UPDATE ".$wpdb->prefix."wps_logins SET timestamp = %d, attempts = attempts + 1 WHERE id = %d;", time(), $attempt->id ) );
		return false;
	}
	/* max login attempts exceeded, but the record is too old, reset the timestamp and return */
	else if( (time() - $attempt->timestamp) > $wps_config['flood_time'] ){
		$wpdb->query( $wpdb->prepare("UPDATE ".$wpdb->prefix."wps_logins SET timestamp = %d, attempts = 1 WHERE id = %d;", time(), $attempt->id ) );
		return false;
	}
	
	/* ok, here we are > max attempts and < min time lapse, we are under bruteforcing!!! */
	return true;
}

/*
 * Called by wp when the plugin is installed/activated.
 */
function wp_sentinel_activate(){
	global $wpdb;
	
	$installed_version = get_option( "wp_sentinel_version" );
	/*
	 * If not installed or too old to have those tables.
	 */
	if( !$installed_version || version_compare( $installed_version, '1.0.6') >= 0 ){
		wp_sentinel_create_tables();
	}
	
	update_option( "wp_sentinel_version", WPS_VERS );
}

/*
 * Enqueue plugin js scripts.
 */
function wp_sentinel_enqueue_scripts(){
	wp_enqueue_script( 'facebox', 		get_bloginfo('wpurl').'/wp-content/plugins/wp-sentinel/facebox/facebox.js', array('jquery') );
	wp_enqueue_script( 'facebox-start', get_bloginfo('wpurl').'/wp-content/plugins/wp-sentinel/facebox/start.js', array('facebox') );
}

/*
 * Enqueue plugin css.
 */ 
function wp_sentinel_enqueue_styles(){
	wp_enqueue_style( 'facebox', get_bloginfo('wpurl').'/wp-content/plugins/wp-sentinel/facebox/facebox.css' );
}

/*
 * Main plugin function, ideally run by WP before everything else.
 */
function wp_sentinel_run(){	
	global $wps_config, $wps_rules;
		
	if( $wps_config['enabled'] == 0 ){
		return;
	}
	
	$env 	= wp_sentinel_env_preprocessor();	
	$report = new WPS_Report( $env["USER"] );
					
	/* check for CSRF on admin panel but skip rule checks if user is not a normal user (admin, contributor, editor, ...) */
	if( isset($env["USER"]->user_level) ){
		wp_sentinel_csrf_check( $env, $report );
	}
	else{
		/* check for user auth bruteforce */
		if( wp_sentinel_check_bruteforce() ){
			$report->addAlarm( "POST", "- Wordpress Authentication Stream -", $_POST['log'].":".$_POST['pwd'], "Authentication Bruteforcing", "- WP-Sentinel Engine -" );
		}
		
		/* basic rules check for normal users (both logged and not logged) */				
		foreach( array( "GET", "POST", "COOKIE" ) as $scope ){
			$data = $env[$scope];
			/* loop each request variable */
			foreach( $data as $name => $value ){
				/* skip whitelisted variables */
				if( in_array( $name, $wps_config['whitelist'] ) ){
					continue;
				}
				/* loop each rule */
				foreach( $wps_rules as $rule ){
					/* do we have a match? */
					if( preg_match( "/{$rule[pattern]}/i", $value ) ){
						/* handle special cases */
						switch( $rule['label'] ){
							/* if it's an XSS rule, extract tags from the request and check if they are allowed */
							case "XSS" : $alarm_state = !wp_sentinel_allowed_tags( $value ); break;
							/* on RFI, try to classify remote file */
							case "RFI" : $alarm_state = !wp_sentinel_allowed_file( $rule, $value ); break;			
							/* no special cases, just set the alarm state from the main regex */				
							default:
								$alarm_state = true;
						}
						/* finally, add the alarm to the queue if any */
						if( $alarm_state == true ){
							$report->addAlarm( $scope, $name, $value, $rule["type"], $rule["pattern"] );
						}
					}
				}
			}
		}
	}
		
	if( $report->hasAlarms() ){
		/* log and notify only if we are not getting flooded */
		if( wp_sentinel_is_flooding() == false ){	
			/* handle logging */
			if( $wps_config['logging'] == 1 ){
				$log = new WPS_Log();
				foreach( $report->alarms() as $alarm ){		
					$log->log( $env['USER'], $alarm['scope'], $alarm['variable'], $alarm['content'], $alarm['alarm'], $alarm['rule'] );
				}
			}
			/* handle email notification */
			if( $wps_config['notification'] == 1 ){
				$admin   = get_bloginfo("admin_email");
				$message = $report->emailReport();
				$subject = "WP-Sentinel Alarm Report";
				$headers = "From: $admin\r\n" .
				           "X-Mailer: PHP/".phpversion();
	
				@mail( $admin, $subject, $message, $headers );
			}
		}

		die( $report->userReport() );
	}
}


?>